use crate::{parsing_utils, solutions::day5::range_mapping::RangeMapping};

use super::mapping_layer::MappingLayer;

pub fn parse(input: &str) -> (Vec<u32>, Vec<MappingLayer>) {
    fn parse_one_layer(lines: Vec<&str>) -> MappingLayer {
        const ALREADY_GOT_MAPPING_NAME: usize = 1;
        let mapping_name = *lines.first().unwrap();
        let mut mappings = Vec::new();

        for next_line in lines.into_iter().skip(ALREADY_GOT_MAPPING_NAME) {
            let next_mapping: RangeMapping = next_line.parse().unwrap();
            mappings.push(next_mapping);
        }

        MappingLayer::new(mapping_name.to_string(), mappings)
    }
    let mut blocks = parsing_utils::blocks_of_lines_seperated_by_empty_lines(input);
    let first_block = blocks.next().unwrap();

    assert!(first_block.len() == 1);
    let seeds = first_block
        .first()
        .unwrap()
        .trim_start_matches("seeds: ")
        .split(" ")
        .map(|to_number| to_number.parse().unwrap())
        .collect();
    let mappings = blocks.into_iter().map(parse_one_layer).collect();

    (seeds, mappings)
}

#[cfg(test)]
mod testing {

    #[test]
    fn parse_input_day5() {
        let input = include_str!("day5_example_input.txt");
        let actual = super::parse(input);
        let expected_seeds: Vec<u32> = vec![79, 14, 55, 13];
        assert_eq!(expected_seeds, actual.0);
        insta::assert_debug_snapshot!(actual.1);
    }
}
